%==================================================================================
function [P, V, F]= Phase_A_Fit( time, voltage, t1, t2, baseline )
% Estimate the parameters (coefficients) of the nonlinear ODE for Phase A
% time:       time vector of the signal (SCALE IS 'MS', i.e. 0ms, 1ms, 2ms, ...
% voltage:    position data of the probe/needle
% -- time and voltage must be column vectors
% t1:         time of the first crest in the data
% t2:         time of the second crest in the data
% -- from t1 and t2, a rough estimate of the decay rate and period can be computed,
%    whereby all subsequent initial parameters can be computed
% baseline:   guess for the steady-state voltage for Phase A
%==================================================================================

    % Reconstructed Voltage
    global V; 
    
    % Estimate parameters
    Mass    = 10;
    T       = t2-t1;                               % Period
    Omega   = 2*pi/T;                              % Frequency  
    Delta   = -1/T * log(voltage(t2)/voltage(t1)); % Decay rate
    Beta    = 2*Mass*Delta;                        % Damping coefficient
    Alpha   = Mass*(Omega^2 + Delta^2);            % Restoring force coeffient
    Shift   = baseline;                            % Steady-state voltage
    A       = voltage(1)-baseline;                 % Initial amplitude for cos(omega t)
    B       = 0.1;                                 % Initial amplitide for sin(omega t)
    P0      = [Delta Omega Shift A B];             % Consolidate initial parameters into one vector

    %Find Optimal Parameters
    P = nlinfit(time,voltage,@Phase_A_NLObj,P0);
  
    % Unroll the parameters
    Delta = P(1); 
    Omega = P(2);  
    Shift = P(3); 
    A     = P(4); 
    B     = P(5);

    % Reconstruct the voltage from the estimated parameters
    V = exp(-Delta*time).*( A *cos(Omega*time) + B*sin(Omega*time) ) + Shift;
     
    % Compute the sum of squares error term (not normalized to the length of the vector 
    F = sum((V-voltage).^2);
    
return    

